/*

  Name: OpenCLFFT

  Author: Uldis Locans

  Info:Extend OpenCLBase class to implement fft and ifft functions using OpenCL

  Data: 19.09.2014

*/
#ifndef H_OPENCL_FFT
#define H_OPENCL_FFT


#include <iostream>
#include <math.h>
#include <complex>

#include "../Algorithms/FFT.h"
#include "OpenCLBase.h"

class OpenCLFFT : public DKSFFT {

private:

  OpenCLBase *m_oclbase;

  /*
    Info: call fft kernels to execute FFT of the given domain,
    data - devevice memory ptr, cdim - current dim to transform, 
    ndim - totla number of dimmensions, N - size of dimension
    Return: success or error code
  */
  int ocl_callFFTKernel(cl_mem &data, int cdim, int ndim, int N, bool forward = true);
	
  /*
    Info: call ifft kernel to execute the bit reverse sort
    data - devevice memory ptr, cdim - current dim to transform, 
    ndim - totla number of dimmensions, N - size of dimension
    Return: success or error code
  */
  int ocl_callBitReverseKernel(cl_mem &data, int cdim, int ndim, int N);

public:

  /* constructor - currently does nothing*/
  OpenCLFFT(OpenCLBase *base) {
    m_oclbase = base;
  }
	
  /* destructor - currently does nothing*/
  ~OpenCLFFT() { }
	
  /*
    Info: execute forward fft function with data set on device
    Return: success or error code
  */
  //int ocl_executeFFT(cl_mem &data, int ndim, int N, bool forward = true);
  int executeFFT(void *data, int ndim, int N[3], int streamId = -1, bool forward = true);
	
  /*
    Info: execute inverse fft with data set on device
    Return: success or error code
  */
  //int ocl_executeIFFT(cl_mem &data, int ndim, int N);
  int executeIFFT(void *data, int ndim, int N[3], int streamId = -1);
	
  /*
    Info: execute normalize kernel
    Return: success or error code
  */
  //int ocl_normalizeFFT(cl_mem &data, int ndim, int N);
  int normalizeFFT(void *data, int ndim, int N[3], int streamId = -1);
	
  /*
    Info: set FFT size
    Return: success or error code
  */
  int setupFFT(int ndim, int N[3]) { return DKS_SUCCESS; }

  int setupFFTRC(int ndim, int N[3], double scale = 1.0) { return DKS_SUCCESS; }

  int setupFFTCR(int ndim, int N[3], double scale = 1.0) { return DKS_SUCCESS; }

  int destroyFFT() { return DKS_SUCCESS; }
	
  int executeRCFFT(void * real_ptr, void * comp_ptr, int ndim, int N[3], 
				int streamId = -1)
    {
      return DKS_ERROR;
    }
  int executeCRFFT(void * real_ptr, void * comp_ptr, int ndim, int N[3], 
				int streamId = -1)
    {
      return DKS_ERROR;
    }
  int normalizeCRFFT(void *real_ptr, int ndim, int N[3], int streamId = -1)
    {
      return DKS_ERROR;
    }

  int ocl_executeFFTStockham(void* &src, int ndim, int N, bool forward = true);

  int ocl_executeFFTStockham2(void* &src, int ndim, int N, bool forward = true);

  int ocl_executeTranspose(void *src, int N[3], int ndim, int dim);
	
  //void printData3DN4(cl_double2* &data, int N);

};

#endif
